@@ -12,8 +12,8 @@ https://docs.djangoproject.com/en/1.6/ref/settings/
 import os
 import sys
 BASE_DIR = os.path.dirname(os.path.dirname(__file__))
-import pymysql
-pymysql.install_as_MySQLdb()
+#import pymysql
+#pymysql.install_as_MySQLdb()
 
 # Quick-start development settings - unsuitable for production
 # See https://docs.djangoproject.com/en/1.6/howto/deployment/checklist/
@@ -26,9 +26,10 @@ DEBUG = False
 
 TEMPLATE_DEBUG = DEBUG
 
-TESTING_MODE = 'test' in sys.argv
+TESTING_MODE = "test" in sys.argv
 
-MAX_THREADS = 4
+MAX_THREADS = 2
+RESPONSE_TIMEOUT = 200./1000.
 
 ALLOWED_HOSTS = ["*"]
 
@@ -39,10 +40,11 @@ INSTALLED_APPS = ([
     "django.contrib.admin",
     "django.contrib.auth",
     "django.contrib.contenttypes",
-    #"django.contrib.sessions",
-    #"django.contrib.messages",
-    #"django.contrib.staticfiles",
+    "django.contrib.sessions",
+    "django.contrib.messages",
+    "django.contrib.staticfiles",
 ] if DEBUG else []) + [
+    "corsheaders",
     "recommendation",
     "recommendation.api",
     "recommendation.filter_owned",
@@ -59,18 +61,24 @@ if TESTING_MODE:
     ]
 
 MIDDLEWARE_CLASSES = (
-    #"django.contrib.sessions.middleware.SessionMiddleware",
-    #"django.middleware.common.CommonMiddleware",
-    #"django.middleware.csrf.CsrfViewMiddleware",
-    #"django.contrib.auth.middleware.AuthenticationMiddleware",
-    #"django.contrib.messages.middleware.MessageMiddleware",
-    #"django.middleware.clickjacking.XFrameOptionsMiddleware",
-    #"django.middleware.transaction.TransactionMiddleware",
+    "django.contrib.sessions.middleware.SessionMiddleware",
+    "corsheaders.middleware.CorsMiddleware",
+    "django.middleware.common.CommonMiddleware",
+    "django.middleware.csrf.CsrfViewMiddleware",
+    "django.contrib.auth.middleware.AuthenticationMiddleware",
+    "django.contrib.messages.middleware.MessageMiddleware",
+    "django.middleware.clickjacking.XFrameOptionsMiddleware",
+    "django.middleware.transaction.TransactionMiddleware",
     #"django.middleware.cache.UpdateCacheMiddleware",
     #"django.middleware.cache.FetchFromCacheMiddleware",
-    #"debug_toolbar.middleware.DebugToolbarMiddleware",
+    "debug_toolbar.middleware.DebugToolbarMiddleware",
+) if DEBUG else (
+    "corsheaders.middleware.CorsMiddleware",
+    "django.middleware.common.CommonMiddleware",
 )
 
+CORS_ORIGIN_ALLOW_ALL = True
+
 ROOT_URLCONF = "recommendation.urls"
 
 WSGI_APPLICATION = "recommendation.wsgi.application"
@@ -78,13 +86,13 @@ WSGI_APPLICATION = "recommendation.wsgi.application"
 
 # Database
 # https://docs.djangoproject.com/en/1.6/ref/settings/#databases
-
 DATABASES = {
     "default": {
-        "ENGINE": "django.db.backends.sqlite3",
-        "NAME": os.path.join(BASE_DIR, "recommender.db"),
-        "ATOMIC_REQUESTS": True,
-        "TEST_NAME": "test_default.db"
+        "ENGINE": "django.db.backends.mysql",
+        "NAME": "ffos",
+        "USER": "root",
+        "PASSWORD": os.environ.get("FRAPPE_PASSWORD", ""),
+        "HOST": os.environ.get("FRAPPE_DB_NAME", "localhost")
     }
 }
 
@@ -109,6 +117,11 @@ STATIC_URL = "/static/"
 # Nose settings
 
 TEST_RUNNER = "django_nose.NoseTestSuiteRunner"
+NOSE_ARGS = [
+    "--with-coverage",
+    "--cover-package=recommendation",
+    "--cover-html"
+]
 #DEBUG_TOOLBAR_PATCH_SETTINGS = False
 
 DEBUG_TOOLBAR_PANELS = (
@@ -140,16 +153,13 @@ CACHES = {
     "default": {
         "BACKEND": "django.core.cache.backends.memcached.MemcachedCache",
         "LOCATION": "127.0.0.1:11211",
-
-    } if not TESTING_MODE else {
+    },
+    "local": {
         "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
         "LOCATION": "django_default_cache",
-        "OPTIONS": {
-            "MAX_ENTRIES": 1000000
-        }
+        "OPTIONS": {"MAX_ENTRIES": 10000000}
     }
 }
-
 # Settings for the recommendation
 
 RECOMMENDATION_SETTINGS = {
@@ -166,4 +176,4 @@ RECOMMENDATION_SETTINGS = {
     },
     "logger": "recommendation.simple_logging.decorators.LogEvent"
     #"logger": "recommendation.decorators.NoLogger"
-}
\ No newline at end of file
+}
