@@ -27,12 +27,11 @@ class SimpleLocaleFilter(object):
         :return: A new set of recommendations ready to fill every item need for the user.
         :rtype: A list of items ids(int).
         """
-        unsupported_langs = Locale.user_locales[user.pk].symmetric_difference(l.pk for l in Locale.all_locales)
+        unsupported_langs = Locale.get_user_locales(user.pk).symmetric_difference(Locale.get_all_locales())
         #print list(Locale.items_by_locale[l] for l in unsupported_langs)
-        unsupported_items = set(chain(*(Locale.items_by_locale[l] for l in unsupported_langs)))
-
+        unsupported_items = set(chain(*(Locale.get_items_by_locale(l) for l in unsupported_langs)))
         for item in unsupported_items:
-            if not any(x in Locale.item_locales[item] for x in Locale.user_locales[user.pk]):
+            if not any(x in Locale.get_item_locales(item) for x in Locale.get_user_locales(user.pk)):
                 early_recommendation[item-1] = float("-inf")
         return early_recommendation
 
